@@ -2,7 +2,7 @@
 #this repository contains the full copyright notices and license terms.
 import copy
 from trytond.model import ModelView, ModelSQL, fields
-from trytond.wizard import Wizard
+from trytond.wizard import Wizard, StateView, Button, StateTransition
 from trytond.report import Report
 from trytond.pyson import Eval, If
 from trytond.transaction import Transaction
@@ -74,32 +74,43 @@ class Employee(ModelSQL, ModelView):
 Employee()
 
 
+class UserEmployee(ModelSQL):
+    'User - Employee'
+    _name = 'res.user-company.employee'
+    _description = __doc__
+    user = fields.Many2One('res.user', 'User', ondelete='CASCADE', select=True,
+        required=True)
+    employee = fields.Many2One('company.employee', 'Employee',
+        ondelete='CASCADE', select=True, required=True)
+
+UserEmployee()
+
+
 class User(ModelSQL, ModelView):
     _name = 'res.user'
     main_company = fields.Many2One('company.company', 'Main Company',
-            on_change=['main_company'])
+        on_change=['main_company'])
     company = fields.Many2One('company.company', 'Current Company',
-            domain=[('parent', 'child_of', [Eval('main_company')], 'parent')],
-            depends=['main_company'])
+        domain=[('parent', 'child_of', [Eval('main_company')], 'parent')],
+        depends=['main_company'], on_change=['company', 'employees'])
     companies = fields.Function(fields.One2Many('company.company', None,
         'Current Companies'), 'get_companies')
+    employees = fields.Many2Many('res.user-company.employee', 'user',
+        'employee', 'Employees')
     employee = fields.Many2One('company.employee', 'Employee',
-        domain=[('company', 'child_of', [Eval('main_company')], 'parent')],
-        depends=['main_company'])
+        domain=[
+            ('company', '=', Eval('company')),
+            ('id', 'in', Eval('employees', [])),
+            ],
+        depends=['company', 'employees'])
 
     def __init__(self):
         super(User, self).__init__()
         self._context_fields.insert(0, 'company')
-        self._constraints += [
-                ('check_company', 'child_company'),
-                ]
-        self._error_messages.update({
-            'child_company': 'You can not set a company that is not ' \
-                    'a child of your main company!',
-        })
+        self._context_fields.insert(0, 'employee')
 
     def default_main_company(self):
-        return Transaction().context.get('company') or False
+        return Transaction().context.get('company')
 
     def default_company(self):
         return self.default_main_company()
@@ -110,7 +121,7 @@ class User(ModelSQL, ModelView):
         company_childs = {}
         for user in self.browse(ids):
             res[user.id] = []
-            company_id = False
+            company_id = None
             if user.company:
                 company_id = user.company.id
             elif user.main_company:
@@ -136,37 +147,61 @@ class User(ModelSQL, ModelView):
         return res
 
     def on_change_main_company(self, vals):
-        return {'company': vals.get('main_company', False)}
+        return {
+            'company': vals.get('main_company'),
+            'employee': None,
+            }
 
-    def check_company(self, ids):
-        company_obj = Pool().get('company.company')
-        for user in self.browse(ids):
-            if user.main_company:
-                companies = company_obj.search([
-                    ('parent', 'child_of', [user.main_company.id]),
+    def on_change_company(self, values):
+        employee_obj = Pool().get('company.employee')
+        result = {
+            'employee': None,
+            }
+        if values.get('company') and values.get('employees'):
+            employee_ids = employee_obj.search([
+                    ('id', 'in', [e['id'] for e in values['employees']]),
+                    ('company', '=', values['company']),
                     ])
-                if user.company.id and (user.company.id not in companies):
-                    return False
-            elif user.company:
-                return False
-        return True
+            if employee_ids:
+                result['employee'] = employee_ids[0]
+        return result
 
     def _get_preferences(self, user, context_only=False):
-        res = super(User, self)._get_preferences(user, 
+        res = super(User, self)._get_preferences(user,
                 context_only=context_only)
         if not context_only:
             res['main_company'] = user.main_company.id
+            if user.main_company.id:
+                res['main_company.rec_name'] = user.main_company.rec_name
+            res['employees'] = [e.id for e in user.employees]
         if user.employee:
             res['employee'] = user.employee.id
+            if user.employee.id:
+                res['employee.rec_name'] = user.employee.rec_name
         return res
 
     def get_preferences_fields_view(self):
-        company_obj = Pool().get('company.company')
-
-        user = self.browse(Transaction().user)
+        pool = Pool()
+        company_obj = pool.get('company.company')
 
         res = super(User, self).get_preferences_fields_view()
         res = copy.deepcopy(res)
+
+        def convert2selection(definition, name):
+            del definition[name]['relation']
+            definition[name]['type'] = 'selection'
+            selection = []
+            definition[name]['selection'] = selection
+            return selection
+
+        if 'company' in res['fields']:
+            selection = convert2selection(res['fields'], 'company')
+            user = self.browse(Transaction().user)
+            company_ids = company_obj.search([
+                    ('parent', 'child_of', [user.main_company.id], 'parent'),
+                    ])
+            for company in company_obj.browse(company_ids):
+                selection.append((company.id, company.rec_name))
         return res
 
     def read(self, ids, fields_names=None):
@@ -213,24 +248,22 @@ class Property(ModelSQL, ModelView):
                 Eval('context', {}).get('company', 0)),
             ])
 
-    def _set_values(self, name, model, res_id, val, field_id):
+    def _set_values(self, model, res_id, val, field_id):
         user_obj = Pool().get('res.user')
         user_id = Transaction().user
         if user_id == 0:
             user_id = Transaction().context.get('user', user_id)
         user = user_obj.browse(user_id)
-        res = super(Property, self)._set_values(name, model, res_id, val, 
-                field_id)
+        res = super(Property, self)._set_values(model, res_id, val, field_id)
         if user:
             res['company'] = user.company.id
         return res
 
-    def search(self, domain, offset=0, limit=None, order=None, count=False,
-            query_string=False):
+    def search(self, domain, offset=0, limit=None, order=None, count=False):
         if Transaction().user == 0 and not 'user' in Transaction().context:
-            domain = ['AND', domain[:], ('company', '=', False)]
-        return super(Property, self).search(domain, offset=offset, limit=limit,
-            order=order, count=count, query_string=query_string)
+            domain = ['AND', domain[:], ('company', '=', None)]
+        return super(Property, self).search(domain, offset=offset,
+                limit=limit, order=order, count=count)
 
 Property()
 
@@ -248,7 +281,7 @@ class Sequence(ModelSQL, ModelView):
         self._order.insert(0, ('company', 'ASC'))
 
     def default_company(self):
-        return Transaction().context.get('company') or False
+        return Transaction().context.get('company')
 
 Sequence()
 
@@ -259,60 +292,56 @@ class SequenceStrict(Sequence):
 SequenceStrict()
 
 
-class CompanyConfigInit(ModelView):
-    'Company Config Init'
-    _name = 'company.company.config.init'
+class CompanyConfigStart(ModelView):
+    'Company Config'
+    _name = 'company.company.config.start'
     _description = __doc__
 
-CompanyConfigInit()
+CompanyConfigStart()
 
 
 class CompanyConfig(Wizard):
-    'Configure companies'
+    'Configure Company'
     _name = 'company.company.config'
-    states = {
-        'init': {
-            'result': {
-                'type': 'form',
-                'object': 'company.company.config.init',
-                'state': [
-                    ('end', 'Cancel', 'tryton-cancel'),
-                    ('company', 'Ok', 'tryton-ok', True),
-                ],
-            },
-        },
-        'company': {
-            'result': {
-                'type': 'form',
-                'object': 'company.company',
-                'state': [
-                    ('end', 'Cancel', 'tryton-cancel'),
-                    ('add', 'Add', 'tryton-ok', True),
-                ],
-            },
-        },
-        'add': {
-            'result': {
-                'type': 'action',
-                'action': '_add',
-                'state': 'end',
-            },
-        },
-    }
 
-    def _add(self, data):
+    start = StateView('company.company.config.start',
+        'company.company_config_start_view_form', [
+            Button('Cancel', 'end', 'tryton-cancel'),
+            Button('Ok', 'company', 'tryton-ok', True),
+            ])
+    company = StateView('company.company',
+        'company.company_view_form', [
+            Button('Cancel', 'end', 'tryton-cancel'),
+            Button('Add', 'add', 'tryton-ok', True),
+            ])
+    add = StateTransition()
+
+    def transition_add(self, session):
         company_obj = Pool().get('company.company')
         user_obj = Pool().get('res.user')
 
-        company_id = company_obj.create(data['form'])
+        values = session.data['company'].copy()
+        for fname in values.keys():
+            if fname in ('id', 'party'):
+                del values[fname]
+                continue
+            if fname in company_obj._columns:
+                field = company_obj._columns[fname]
+            else:
+                field = company_obj._inherit_fields[fname][2]
+            if field._type == 'one2many':
+                values[fname] = [('create', v) for v in values[fname]]
+            elif field._type == 'many2many':
+                values[fname] = [('set', [v['id'] for v in values[fname]])]
+        company_id = company_obj.create(values)
         user_ids = user_obj.search([
-            ('main_company', '=', False),
+            ('main_company', '=', None),
             ])
         user_obj.write(user_ids, {
             'main_company': company_id,
             'company': company_id,
             })
-        return {}
+        return 'end'
 
 CompanyConfig()
 
